<?php

namespace Drupal\image_webp_converter\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use WebPConvert\WebPConvert;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\file\Entity\File;

/**
 * Implements a form to upload an image, convert to WebP, and allow download.
 */
class UploadWebpConverter extends FormBase {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * Constructs an UploadWebpConverter form.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   */
  public function __construct(FileSystemInterface $file_system, MessengerInterface $messenger, FileUrlGeneratorInterface $file_url_generator) {
    $this->fileSystem = $file_system;
    $this->messenger = $messenger;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('messenger'),
      $container->get('file_url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'image_webp_converter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Upload Image'),
      '#description' => $this->t('Allowed file types: jpg, jpeg, png'),
      '#upload_location' => 'public://converted_images/',
      '#required' => TRUE,
      '#validators' => [
        'file_validate_extensions' => ['jpg jpeg png'],
      ],
    ];

    $form['note'] = [
      '#type' => 'markup',
      '#markup' => $this->t('<p><strong>Note:</strong> After uploading and conversion, you will receive a WebP download link.</p>'),
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Upload and Convert'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $file_id = $form_state->getValue('image');

    if (empty($file_id[0])) {
      $this->messenger->addError($this->t('Failed to process the uploaded file. Please try again.'));
      return;
    }

    $file = File::load($file_id[0]);

    if (!$file) {
      $this->messenger->addError($this->t('Failed to load the uploaded file. Please try again.'));
      return;
    }

    // Marks the file as temporary so Drupal can clean it up later.
    $file->setTemporary();
    $file->save();

    $file_uri = $file->getFileUri();
    $absolute_path = $this->fileSystem->realpath($file_uri);

    try {
      // Converts the image to WebP.
      $webp_file_uri = $this->convertToWebP($absolute_path);

      // Checks if the WebP file exists before generating a URL.
      if (!file_exists($this->fileSystem->realpath($webp_file_uri))) {
        throw new \Exception('WebP file not found at ' . $webp_file_uri);
      }

      // Generate the public URL for the WebP file.
      $webp_url = $this->fileUrlGenerator->generateAbsoluteString($webp_file_uri);

      $this->messenger->addMessage($this->t('Image converted successfully. <a href=":link" download>Download WebP image</a>.', [':link' => $webp_url]));

    }
    catch (\Exception $e) {
      $this->messenger->addError($this->t('An error occurred during conversion: @message', ['@message' => $e->getMessage()]));
    }
  }

  /**
   * Converts an image to WebP format and returns the file URI.
   *
   * @param string $file_path
   *   The absolute path of the uploaded image.
   *
   * @return string
   *   The URI of the converted WebP image.
   *
   * @throws \Exception
   *   If the conversion fails.
   */
  private function convertToWebP($file_path) {
    if (!$file_path || !file_exists($file_path)) {
      throw new \Exception('Source file does not exist: ' . $file_path);
    }

    // Ensures the output directory exists.
    $webp_dir = 'public://converted_images/';
    $this->fileSystem->prepareDirectory($webp_dir, FileSystemInterface::CREATE_DIRECTORY);

    // Define the WebP file path.
    $webp_file_uri = $webp_dir . pathinfo($file_path, PATHINFO_FILENAME) . '.webp';
    $webp_absolute_path = $this->fileSystem->realpath($webp_file_uri);

    // Converts the image to WebP.
    try {
      WebPConvert::convert($file_path, $webp_absolute_path);
    }
    catch (\Exception $e) {
      throw new \Exception('WebP conversion failed: ' . $e->getMessage());
    }

    // Ensure the WebP file exists.
    if (!file_exists($webp_absolute_path)) {
      throw new \Exception('WebP file was not created successfully.');
    }

    return $webp_file_uri;
  }

}
